/****************************************************************************
  Interfacing PIC18F46K22 microcontroller with Nokia 5110 (3310) LCD display.
  Graphics test example.
  C Code for mikroC PRO for PIC compiler.
  Internal oscillator used @ 16MHz
  Configuration words: CONFIG1H = 0x0028
                       CONFIG2L = 0x0018
                       CONFIG2H = 0x003C
                       CONFIG3H = 0x0037
                       CONFIG4L = 0x0081
                       CONFIG5L = 0x000F
                       CONFIG5H = 0x00C0
                       CONFIG6L = 0x000F
                       CONFIG6H = 0x00E0
                       CONFIG7L = 0x000F
                       CONFIG7H = 0x0040
  This is a free software with NO WARRANTY.
  https://simple-circuit.com/

*****************************************************************************
  This is an example sketch for our Monochrome Nokia 5110 LCD Displays

  Pick one up today in the adafruit shop!
  ------> http://www.adafruit.com/products/338

  These displays use SPI to communicate, 4 or 5 pins are required to
  interface

  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Limor Fried/Ladyada  for Adafruit Industries.
  BSD license, check license.txt for more information
  All text above, and the splash screen must be included in any redistribution
*****************************************************************************/

// define Nokia LCD module pin connections
// RST is optional!
#define LCD_RST  RD0_bit
#define LCD_CS   RD1_bit
#define LCD_DC   RD2_bit
#define LCD_DAT  RD3_bit
#define LCD_CLK  RD4_bit
#define LCD_RST_DIR   TRISD0_bit
#define LCD_CS_DIR    TRISD1_bit
#define LCD_DC_DIR    TRISD2_bit
#define LCD_DAT_DIR   TRISD3_bit
#define LCD_CLK_DIR   TRISD4_bit

#include "PCD8544.c"      // include PCD8544 controller driver source code (Nokia 5110 LCD driver)
#include "GFX_Library.c"  // include graphics library source code

#define NUMFLAKES 10
#define XPOS 0
#define YPOS 1
#define DELTAY 2

#define LOGO16_GLCD_HEIGHT 16
#define LOGO16_GLCD_WIDTH  16
const char logo16_glcd_bmp[] =
{ 0b00000000, 0b11000000,
  0b00000001, 0b11000000,
  0b00000001, 0b11000000,
  0b00000011, 0b11100000,
  0b11110011, 0b11100000,
  0b11111110, 0b11111000,
  0b01111110, 0b11111111,
  0b00110011, 0b10011111,
  0b00011111, 0b11111100,
  0b00001101, 0b01110000,
  0b00011011, 0b10100000,
  0b00111111, 0b11100000,
  0b00111111, 0b11110000,
  0b01111100, 0b11110000,
  0b01110000, 0b01110000,
  0b00000000, 0b00110000 };


void testdrawbitmap(const uint8_t *bitmap, uint8_t w, uint8_t h) {
  static uint8_t icons[NUMFLAKES][3], f, x_pos, y_pos;

  // initialize
  for (f=0; f< NUMFLAKES; f++) {
    icons[f][XPOS] = rand();
    icons[f][YPOS] = 0;
    icons[f][DELTAY] = (rand() % 5) + 1;
  }
  icons[0][XPOS] = 30;

  while (1) {
    uint8_t f;
    // draw each icon
    for (f=0; f< NUMFLAKES; f++) {
      x_pos = icons[f][XPOS];
      y_pos = icons[f][YPOS];
      display_drawBitmapV2(x_pos, y_pos, bitmap, w, h, BLACK);
    }
    display();
    delay_ms(200);

    // then erase it + move it
    for (f=0; f< NUMFLAKES; f++) {
      x_pos = icons[f][XPOS];
      y_pos = icons[f][YPOS];
      display_drawBitmapV2(x_pos, y_pos, bitmap, w, h, WHITE);
      // move it
      icons[f][YPOS] += icons[f][DELTAY];
      // if its gone, reinit
      if (icons[f][YPOS] > display_height) {
        icons[f][XPOS] = rand();
        icons[f][YPOS] = 0;
        icons[f][DELTAY] = (rand() % 5) + 1;
      }
    }
   }
}

void testdrawchar(void) {
  char i;
  display_setTextSize(1);
  display_setTextColor(BLACK, WHITE);
  display_setCursor(0, 0);

  for (i=0; i < 168; i++) {
    if (i == '\n' || i == '\r') continue;
    display_putc(i);
  }
  display();
}

void testdrawcircle(void) {
  char i;
  for (i=0; i<display_height; i+=2) {
    display_drawCircle(display_width/2, display_height/2, i, BLACK);
    display();
  }
}

void testfillrect(void) {
  char i, color = 1;
  for (i=0; i<display_height/2; i+=3) {
    // alternate colors
    display_fillRect(i, i, display_width-i*2, display_height-i*2, color%2);
    display();
    color++;
  }
}

void testdrawtriangle(void) {
  char i;
  for (i=0; i<min(display_width,display_height)/2; i+=5) {
    display_drawTriangle(display_width/2, display_height/2-i,
                     display_width/2-i, display_height/2+i,
                     display_width/2+i, display_height/2+i, BLACK);
    display();
  }
}

void testfilltriangle(void) {
  short i, color = BLACK;
  for (i=min(display_width,display_height)/2; i>0; i-=5) {
    display_fillTriangle(display_width/2, display_height/2-i,
                     display_width/2-i, display_height/2+i,
                     display_width/2+i, display_height/2+i, color);
    if (color == WHITE) color = BLACK;
    else color = WHITE;
    display();
  }
}

void testdrawroundrect(void) {
  char i;
  for (i=0; i<display_height/2-2; i+=2) {
    display_drawRoundRect(i, i, display_width-2*i, display_height-2*i, display_height/4, BLACK);
    display();
  }
}

void testfillroundrect(void) {
  char i, color = BLACK;
  for (i=0; i<display_height/2-2; i+=2) {
    display_fillRoundRect(i, i, display_width-2*i, display_height-2*i, display_height/4, color);
    if (color == WHITE) color = BLACK;
    else color = WHITE;
    display();
  }
}

void testdrawrect(void) {
  char i;
  for (i=0; i<display_height/2; i+=2) {
    display_drawRect(i, i, display_width-2*i, display_height-2*i, BLACK);
    display();
  }
}

void testdrawline() {
  short i;
  for (i=0; i<display_width; i+=4) {
    display_drawLine(0, 0, i, display_height-1, BLACK);
    display();
  }
  for (i=0; i<display_height; i+=4) {
    display_drawLine(0, 0, display_width-1, i, BLACK);
    display();
  }
  delay_ms(250);

  display_clear();
  for (i=0; i<display_width; i+=4) {
    display_drawLine(0, display_height-1, i, 0, BLACK);
    display();
  }
  for (i=display_height-1; i>=0; i-=4) {
    display_drawLine(0, display_height-1, display_width-1, i, BLACK);
    display();
  }
  delay_ms(250);

  display_clear();
  for (i=display_width-1; i>=0; i-=4) {
    display_drawLine(display_width-1, display_height-1, i, 0, BLACK);
    display();
  }
  for (i=display_height-1; i>=0; i-=4) {
    display_drawLine(display_width-1, display_height-1, 0, i, BLACK);
    display();
  }
  delay_ms(250);

  display_clear();
  for (i=0; i<display_height; i+=4) {
    display_drawLine(display_width-1, 0, 0, i, BLACK);
    display();
  }
  for (i=0; i<display_width; i+=4) {
    display_drawLine(display_width-1, 0, i, display_height-1, BLACK);
    display();
  }
  delay_ms(250);
}

// main function
void main() {
  
  OSCCON = 0x70;   // set internal oscillator to 16MHz
  ANSELD = 0;      // configure all PORTD pins as digital

  // initialize the LCD
  LCD_begin();
  // init done
  
  // you can change the contrast around (0 ... 63) to
  //adapt the display for the best viewing!
  display_setContrast(50);

  display(); // show splashscreen
  delay_ms(2000);
  display_clear();   // clears the screen and buffer

  // draw a single pixel
  display_drawPixel(10, 10, BLACK);
  display();
  delay_ms(2000);
  display_clear();

  // draw many lines
  testdrawline();
  display();
  delay_ms(2000);
  display_clear();

  // draw rectangles
  testdrawrect();
  display();
  delay_ms(2000);
  display_clear();

  // draw multiple rectangles
  testfillrect();
  display();
  delay_ms(2000);
  display_clear();

  // draw mulitple circles
  testdrawcircle();
  display();
  delay_ms(2000);
  display_clear();

  // draw a circle, 10 pixel radius
  display_fillCircle(display_width/2, display_height/2, 10, BLACK);
  display();
  delay_ms(2000);
  display_clear();

  testdrawroundrect();
  delay_ms(2000);
  display_clear();

  testfillroundrect();
  delay_ms(2000);
  display_clear();

  testdrawtriangle();
  delay_ms(2000);
  display_clear();

  testfilltriangle();
  delay_ms(2000);
  display_clear();

  // draw the first ~12 characters in the font
  testdrawchar();
  display();
  delay_ms(2000);
  display_clear();

  // text display tests
  display_setTextSize(1);
  display_setTextColor(BLACK, WHITE);
  display_setCursor(0, 0);
  display_puts("Hello, world!\r\n");
  display_setTextColor(WHITE, BLACK); // 'inverted' text
  display_printf("%4.2f\r\n", 3.141592);
  display_setTextSize(2);
  display_setTextColor(BLACK, WHITE);
  display_printf("0x%LX\r\n", 0xDEADBEEF);
  display();
  delay_ms(2000);

  // rotation example
  display_clear();
  display_setRotation(1);  // rotate 90 degrees counter clockwise, can also use values of 2 and 3 to go further.
  display_setTextSize(1);
  display_setTextColor(BLACK, WHITE);
  display_setCursor(0, 0);
  display_puts("Rotation\r\n");
  display_setTextSize(2);
  display_puts("Example!\r\n");
  display();
  delay_ms(2000);

  // revert back to no rotation
  display_setRotation(0);

  // miniature bitmap display
  display_clear();
  display_drawBitmapV2(30, 16,  logo16_glcd_bmp, 16, 16, BLACK);
  display();

  // invert the display
  display_invert(true);
  delay_ms(1000);
  display_invert(false);
  delay_ms(1000);

  // draw a bitmap icon and 'animate' movement
  testdrawbitmap(logo16_glcd_bmp, LOGO16_GLCD_WIDTH, LOGO16_GLCD_HEIGHT);
}

// end of code.